import datetime
import json

import numpy as np
import pandas as pd
import requests


class oandacandle:
    def __init__(self, authorization, endpoint, sym, payload):
        self.authorization = authorization
        self.sym = sym
        self.endpoint = endpoint
        self.payload = payload

    def get(self):

        authorization = self.authorization
        endpoint = self.endpoint
        sym = self.sym
        payload = self.payload

        auth = {"Authorization": "Bearer " + authorization}
        url = endpoint + sym + "/candles"
        error = 0

        try:

            response = requests.get(url, params=payload, headers=auth)
            raw = response.json()
            data = json.dumps(raw, indent=2)
            dataset = pd.read_json(data)
            df = pd.json_normalize(dataset["candles"])
            del df["complete"]

            df["time"] = pd.to_datetime(df["time"]).dt.tz_localize(None)
            df.columns = ["vol", "time", "open", "high", "low", "close"]
            df1 = df[["time", "vol", "high", "low", "open", "close"]]

        except:

            error = 1
            df1 = pd.DataFrame()

        return df1, error


class oandaclose:
    def __init__(self, symbol, account, authorization, los, endpoint):

        self.symbol = symbol
        self.account = account
        self.authorization = authorization
        self.los = los
        self.endpoint = endpoint

    def closeposition(self):

        symbol = self.symbol
        account = self.account
        authorization = self.authorization
        los = self.los

        # Headers and Parameters
        url = self.endpoint + account + "/positions/" + symbol + "/close"
        auth = {
            "Authorization": "Bearer " + authorization,
            "Content-Type": "application/json",
        }

        if los == 0:
            order_details = {"shortUnits": "ALL"}
        else:
            order_details = {"longUnits": "ALL"}

        # Execute Position Close
        response = requests.put(
            url, headers=auth, data=json.dumps(order_details), timeout=5
        )

        if response.status_code == 200:
            close = 1

        else:

            # Position Not Found
            if response.status_code == 404:
                close = 1
            else:
                # WAIT
                print(
                    str(account)
                    + " - WAIT "
                    + str(los)
                    + " "
                    + symbol
                    + " : "
                    + str(response.status_code)
                )
                close = 8

        return close


class oandaorders:
    def __init__(
        self,
        account,
        authorization,
        price,
        stoploss,
        takeprofit,
        symbol,
        units,
        endpoint,
        ttl,
    ):

        self.account = account
        self.authorization = authorization
        self.price = price
        self.stoploss = stoploss
        self.takeprofit = takeprofit
        self.symbol = symbol
        self.units = units
        self.endpoint = endpoint
        self.ttl = ttl

    def postlimit(self):

        # Redundancy
        account = self.account
        authorization = self.authorization
        price = self.price
        stoploss = self.stoploss
        takeprofit = self.takeprofit
        symbol = self.symbol
        units = self.units
        endpoint = self.endpoint

        # Get TTL
        ts = datetime.datetime.now() + datetime.timedelta(seconds=int(self.ttl))
        ttl = round(ts.timestamp(), 2)

        if "JPY" in symbol:

            price = round(price, 2)
            risk_check = round(abs(stoploss), 2)
            profit_check = round(abs(takeprofit), 2)

        else:

            price = round(price, 4)
            risk_check = round(abs(stoploss), 4)
            profit_check = round(abs(takeprofit), 4)

        # Headers and Parameters
        url = endpoint + account + "/orders"
        auth = {
            "Authorization": "Bearer " + authorization,
            "Content-Type": "application/json",
        }

        order_details = {
            "order": {
                "price": str(price),
                "instrument": str(symbol),
                "units": str(units),
                "stopLossOnFill": {"timeInForce": "GTC", "distance": str(risk_check)},
                "takeProfitOnFill": {
                    "timeInForce": "GTC",
                    "distance": str(profit_check),
                },
                "timeInForce": "GTD",
                "gtdTime": str(ttl),
                "type": "LIMIT",
                "positionFill": "DEFAULT",
            }
        }

        # Execute Order
        response = requests.post(
            url, headers=auth, data=json.dumps(order_details), timeout=5
        )

        if response.status_code == 201:

            raw = response.json()
            data = json.dumps(raw, indent=2)
            dataset = json.loads(data)
            ticket = dataset["orderCreateTransaction"]["id"]
            print(
                str(account)
                + " - PLACED "
                + str(units)
                + " "
                + symbol
                + " : "
                + str(response.status_code)
            )

        else:

            print(
                str(account)
                + " - FAILED "
                + str(units)
                + " "
                + symbol
                + " : "
                + str(response.status_code)
            )
            ticket = np.nan
            raw = response.json()
            data = json.dumps(raw, indent=2)
            dataset = json.loads(data)
            reason = dataset["errorMessage"]
            print("LIMIT ORDER")
            print(str(reason))
            print(" ")

        return float(ticket)

    def postmarket(self):

        # Redundancy
        account = self.account
        authorization = self.authorization
        price = self.price
        stoploss = self.stoploss
        takeprofit = self.takeprofit
        symbol = self.symbol
        units = self.units
        endpoint = self.endpoint

        # Get TTL
        ts = datetime.datetime.now() + datetime.timedelta(seconds=int(self.ttl))
        ttl = round(ts.timestamp(), 2)

        if "JPY" in symbol:

            price = round(price, 2)
            risk_check = round(abs(stoploss), 2)
            profit_check = round(abs(takeprofit), 2)

        else:

            price = round(price, 4)
            risk_check = round(abs(stoploss), 4)
            profit_check = round(abs(takeprofit), 4)

        # Headers and Parameters
        url = endpoint + account + "/orders"
        auth = {
            "Authorization": "Bearer " + authorization,
            "Content-Type": "application/json",
        }

        order_details = {
            "order": {
                "priceBound": str(price),
                "instrument": str(symbol),
                "units": str(units),
                "stopLossOnFill": {"timeInForce": "GTC", "distance": str(risk_check)},
                "takeProfitOnFill": {
                    "timeInForce": "GTC",
                    "distance": str(profit_check),
                },
                "timeInForce": "FOK",
                "gtdTime": str(ttl),
                "type": "MARKET",
                "positionFill": "DEFAULT",
            }
        }

        # Execute Order
        response = requests.post(
            url, headers=auth, data=json.dumps(order_details), timeout=5
        )

        if response.status_code == 201:

            raw = response.json()
            data = json.dumps(raw, indent=2)
            dataset = json.loads(data)
            ticket = dataset["orderCreateTransaction"]["id"]
            print(
                str(account)
                + " - PLACED "
                + str(units)
                + " "
                + symbol
                + " : "
                + str(response.status_code)
            )

        else:

            print(
                str(account)
                + " - FAILED "
                + str(units)
                + " "
                + symbol
                + " : "
                + str(response.status_code)
            )
            ticket = np.nan
            raw = response.json()
            data = json.dumps(raw, indent=2)
            dataset = json.loads(data)
            reason = dataset["errorMessage"]
            print("MARKET ORDER")
            print(str(reason))
            print(" ")

        return float(ticket)


class oandadaily:
    def __init__(
        self,
        account,
        authorization,
        endpoint,
    ):

        self.account = account
        self.authorization = authorization
        self.endpoint = endpoint

    def getpnl(self):

        now = datetime.datetime.now()
        posting_date = now.replace(hour=0, minute=0, second=0, microsecond=0)

        url_account = self.endpoint + self.account + "/transactions"
        auth = {
            "Authorization": "Bearer " + self.authorization,
            "Content-Type": "application/json",
        }

        # Get Time
        if posting_date.weekday() == 0:
            from_date = datetime.date.today() - datetime.timedelta(days=3)
        else:
            from_date = datetime.date.today() - datetime.timedelta(days=1)

        to_date = datetime.date.today() + datetime.timedelta(days=1)

        try:

            payload_one = {"from": from_date, "to": to_date, "pageSize": 999}
            response = requests.get(url_account, params=payload_one, headers=auth)
            raw = response.json()
            df = pd.DataFrame(raw["pages"])

            # Get Data
            data_merge = pd.DataFrame()
            for i in range(len(df)):
                url_arr = df.loc[i]
                url = str(url_arr[0])

                response1 = requests.get(url, headers=auth)
                raw = response1.json()
                data = pd.DataFrame(raw["transactions"])
                data_merge = data_merge.append(data)

            if "instrument" in data_merge.columns:
                error = 0
            else:
                error = 1

        except:

            error = 1
            data_merge = pd.DataFrame()

        return data_merge, error



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gbrokerexe']

package_data = \
{'': ['*']}

install_requires = \
['numpy==1.19.5', 'pandas>=1.3.2,<2.0.0', 'pytest>=6.2.4,<7.0.0']

setup_kwargs = {
    'name': 'gbrokerexe',
    'version': '0.1.7',
    'description': 'Generic Python Package for Brokerage and Trade Executions.',
    'long_description': '# gbrokerexe\n\n<div align="right">\n\n[![Build status](https://github.com/altcp/gbrokerexe/workflows/build/badge.svg?branch=master&event=push)](https://github.com/altcp/gbrokerexe/actions?query=workflow%3Abuild)\n[![Dependencies Status](https://img.shields.io/badge/dependencies-up%20to%20date-brightgreen.svg)](https://github.com/altcp/gbrokerexe/pulls?utf8=%E2%9C%93&q=is%3Apr%20author%3Aapp%2Fdependabot)\n[![pypi](https://img.shields.io/pypi/v/gbrokerexe.svg)](https://pypi.python.org/pypi/gbrokerexe)\n\n</div>\n\nGeneric Python Package for Brokerage and Trade Executions.\n<p>&nbsp;</p>\n\n\n\n## Features \n\n<div align="right">\n\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black) \n[![Pre-commit](https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white)](https://github.com/altcp/gbrokerexe/blob/master/.pre-commit-config.yaml)\n\n</div>\n\n* oanda API \n* alpaca-trade API\n* others\n<p>&nbsp;</p>\n\n\n\n## Releases\n\n<div align="right">\n\n[![Release Drafter](https://github.com/altcp/gbrokerexe/actions/workflows/release-drafter.yml/badge.svg)](https://github.com/altcp/gbrokerexe/actions/workflows/release-drafter.yml)\n[![Semantic Versions](https://img.shields.io/badge/%20%20%F0%9F%93%A6%F0%9F%9A%80-semantic--versions-e10079.svg)](https://github.com/altcp/gdemandfcast/releases)\n\n</div>\n\nYou can see the list of available releases on the [GitHub Releases](https://github.com/altcp/gdemandfcast/releases) page. &nbsp;\nWe use [`Release Drafter`](https://github.com/marketplace/actions/release-drafter). As pull requests are merged, a draft release is kept up-to-date listing the changes.\n<p>&nbsp;</p>\n\n\n\n## License\n\n<div align="right">\n\n[![License](https://img.shields.io/github/license/altcp/gbrokerexe)](https://github.com/altcp/gbrokerexe/blob/master/LICENSE)\n\n</div>\n\nThis project is licensed under the terms of the `GNU GPL v3.0` license.\n<p>&nbsp;</p>\n\n\n\n## 📃 Citation\n<p>&nbsp;</p>\n\n```bibtex\n@misc{gbrokerexe,\n  author = {altcp},\n  title = {Generic Python Package for Brokerage and Trade Executions},\n  year = {2021},\n  publisher = {GitHub},\n  journal = {GitHub repository},\n  howpublished = {https://github.com/altcp/gbrokerexe}\n}\n```\n<p>&nbsp;</p>\n\n\n## Credits \n<div align="right">\n\n[![🚀 Your next Python package needs a bleeding-edge project structure.](https://img.shields.io/badge/python--package--template-%F0%9F%9A%80-brightgreen)](https://github.com/TezRomacH/python-package-template)\n\n</div>\n\nThis project was generated with [`python-package-template`](https://github.com/TezRomacH/python-package-template)\n',
    'author': 'altcp',
    'author_email': 'colab.tcp@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/altcp/gbrokerexe',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<3.11',
}


setup(**setup_kwargs)
